@@ -28,17 +28,17 @@
 #include <string.h>
 
 #include "../core/weechat.h"
-#include "../core/wee-arraylist.h"
-#include "../core/wee-config.h"
-#include "../core/wee-debug.h"
-#include "../core/wee-hashtable.h"
-#include "../core/wee-hdata.h"
-#include "../core/wee-hook.h"
-#include "../core/wee-infolist.h"
-#include "../core/wee-log.h"
-#include "../core/wee-string.h"
-#include "../core/wee-utf8.h"
-#include "../core/wee-util.h"
+#include "../core/core-arraylist.h"
+#include "../core/core-config.h"
+#include "../core/core-debug.h"
+#include "../core/core-hashtable.h"
+#include "../core/core-hdata.h"
+#include "../core/core-hook.h"
+#include "../core/core-infolist.h"
+#include "../core/core-log.h"
+#include "../core/core-string.h"
+#include "../core/core-utf8.h"
+#include "../core/core-util.h"
 #include "../plugins/plugin.h"
 #include "gui-bar-item.h"
 #include "gui-bar.h"
@@ -66,8 +66,8 @@ char *gui_bar_item_names[GUI_BAR_NUM_ITEMS] =
   "buffer_name", "buffer_short_name", "buffer_modes", "buffer_filter",
   "buffer_zoom", "buffer_nicklist_count", "buffer_nicklist_count_groups",
   "buffer_nicklist_count_all", "scroll", "hotlist", "completion",
-  "buffer_title", "buffer_nicklist", "window_number", "mouse_status", "away",
-  "spacer"
+  "buffer_title", "buffer_nicklist", "window_number", "mouse_status", "lag",
+  "away", "spacer"
 };
 struct t_gui_bar_item_hook *gui_bar_item_hooks = NULL;
 struct t_hook *gui_bar_item_timer = NULL;
@@ -442,8 +442,7 @@ gui_bar_item_get_value (struct t_gui_bar *bar, struct t_gui_window *window,
                         ptr_item->name,
                         plugin_get_name (ptr_item->plugin),
                         str_diff);
-                    if (str_diff)
-                        free (str_diff);
+                    free (str_diff);
                 }
             }
         }
@@ -459,8 +458,7 @@ gui_bar_item_get_value (struct t_gui_bar *bar, struct t_gui_window *window,
     result = string_dyn_alloc (128);
     if (!result)
     {
-        if (item_value)
-            free (item_value);
+        free (item_value);
         return NULL;
     }
 
@@ -532,8 +530,7 @@ gui_bar_item_get_value (struct t_gui_bar *bar, struct t_gui_window *window,
         string_dyn_concat (result, bar->items_suffix[item][subitem], -1);
     }
 
-    if (item_value)
-        free (item_value);
+    free (item_value);
 
     return string_dyn_free (result, 0);
 }
@@ -730,10 +727,8 @@ gui_bar_item_free (struct t_gui_bar_item *item)
         last_gui_bar_item = item->prev_item;
 
     /* free data */
-    if (item->name)
-        free (item->name);
-    if (item->build_callback_data)
-        free (item->build_callback_data);
+    free (item->name);
+    free (item->build_callback_data);
 
     free (item);
 }
@@ -820,8 +815,6 @@ gui_bar_item_input_prompt_cb (const void *pointer, void *data,
                               struct t_gui_buffer *buffer,
                               struct t_hashtable *extra_info)
 {
-    const char *nick;
-
     /* make C compiler happy */
     (void) pointer;
     (void) data;
@@ -832,9 +825,7 @@ gui_bar_item_input_prompt_cb (const void *pointer, void *data,
     if (!buffer)
         return NULL;
 
-    nick = (const char *)hashtable_get (buffer->local_variables, "nick");
-
-    return (nick) ? strdup (nick) : NULL;
+    return (buffer->input_prompt) ? strdup (buffer->input_prompt) : NULL;
 }
 
 /*
@@ -956,8 +947,7 @@ gui_bar_item_input_text_cb (const void *pointer, void *data,
     }
 
     /* for modifiers */
-    snprintf (str_buffer, sizeof (str_buffer),
-              "0x%lx", (unsigned long)buffer);
+    snprintf (str_buffer, sizeof (str_buffer), "0x%lx", (unsigned long)buffer);
 
     /* execute modifier with basic string (without cursor tag) */
     ptr_input = NULL;
@@ -1017,8 +1007,7 @@ gui_bar_item_input_text_cb (const void *pointer, void *data,
                                          "input_text_display_with_cursor",
                                          str_buffer,
                                          (ptr_input) ? ptr_input : "");
-        if (ptr_input)
-            free (ptr_input);
+        free (ptr_input);
         ptr_input = ptr_input2;
     }
 
@@ -1307,15 +1296,24 @@ gui_bar_item_buffer_modes_cb (const void *pointer, void *data,
                               struct t_gui_buffer *buffer,
                               struct t_hashtable *extra_info)
 {
+    char *modes;
+
     /* make C compiler happy */
     (void) pointer;
     (void) data;
     (void) item;
     (void) window;
-    (void) buffer;
     (void) extra_info;
 
-    return NULL;
+    if (!buffer || !buffer->modes)
+        return NULL;
+
+    string_asprintf (
+        &modes,
+        "%s%s",
+        gui_color_get_custom (gui_color_get_name (CONFIG_COLOR(config_color_status_modes))),
+        buffer->modes);
+    return modes;
 }
 
 /*
@@ -1735,8 +1733,7 @@ gui_bar_item_hotlist_cb (const void *pointer, void *data,
         }
     }
 
-    if (buffer_without_name_displayed)
-        free (buffer_without_name_displayed);
+    free (buffer_without_name_displayed);
 
     hotlist_suffix = CONFIG_STRING(config_look_hotlist_suffix);
     if (hotlist_suffix[0]
@@ -2045,6 +2042,38 @@ gui_bar_item_mouse_status_cb (const void *pointer, void *data,
     return strdup (str_mouse);
 }
 
+/*
+ * Bar item with buffer lag.
+ */
+
+char *
+gui_bar_item_lag_cb (const void *pointer, void *data,
+                     struct t_gui_bar_item *item,
+                     struct t_gui_window *window,
+                     struct t_gui_buffer *buffer,
+                     struct t_hashtable *extra_info)
+{
+    const char *lag;
+    char str_lag[1024];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    lag = (const char *)hashtable_get (buffer->local_variables, "lag");
+    if (!lag)
+        return NULL;
+
+    snprintf (str_lag, sizeof (str_lag), "%s: %s", _("Lag"), lag);
+    return strdup (str_lag);
+}
+
 /*
  * Bar item with away message.
  */
@@ -2258,13 +2287,24 @@ gui_bar_item_signal_cb (const void *pointer, void *data,
                         const char *signal,
                         const char *type_data, void *signal_data)
 {
+    const char *item;
+
     /* make C compiler happy */
     (void) data;
     (void) signal;
     (void) type_data;
     (void) signal_data;
 
-    gui_bar_item_update ((char *)pointer);
+    item = (const char *)pointer;
+    if (item)
+    {
+        if ((strcmp (item, "hotlist") == 0)
+            && (strcmp (signal, "hotlist_changed") != 0))
+        {
+            gui_hotlist_resort ();
+        }
+        gui_bar_item_update (item);
+    }
 
     return WEECHAT_RC_OK;
 }
@@ -2309,7 +2349,8 @@ gui_bar_item_init ()
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_INPUT_PROMPT],
                       &gui_bar_item_input_prompt_cb, NULL, NULL);
-    gui_bar_item_hook_signal ("window_switch;buffer_switch;buffer_localvar_*",
+    gui_bar_item_hook_signal ("window_switch;buffer_switch;input_prompt_changed;"
+                              "buffer_localvar_*",
                               gui_bar_item_names[GUI_BAR_ITEM_INPUT_PROMPT]);
 
     /* input search */
@@ -2387,7 +2428,7 @@ gui_bar_item_init ()
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_BUFFER_MODES],
                       &gui_bar_item_buffer_modes_cb, NULL, NULL);
-    gui_bar_item_hook_signal ("window_switch;buffer_switch",
+    gui_bar_item_hook_signal ("window_switch;buffer_switch;buffer_modes_changed",
                               gui_bar_item_names[GUI_BAR_ITEM_BUFFER_MODES]);
 
     /* buffer filter */
@@ -2437,7 +2478,8 @@ gui_bar_item_init ()
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_HOTLIST],
                       &gui_bar_item_hotlist_cb, NULL, NULL);
-    gui_bar_item_hook_signal ("hotlist_changed;buffer_moved;buffer_closed",
+    gui_bar_item_hook_signal ("hotlist_changed;buffer_moved;buffer_closed;"
+                              "buffer_localvar_*",
                               gui_bar_item_names[GUI_BAR_ITEM_HOTLIST]);
 
     /* completion (possible words when a partial completion occurs) */
@@ -2479,6 +2521,13 @@ gui_bar_item_init ()
     gui_bar_item_hook_signal ("mouse_enabled;mouse_disabled",
                               gui_bar_item_names[GUI_BAR_ITEM_MOUSE_STATUS]);
 
+    /* lag */
+    gui_bar_item_new (NULL,
+                      gui_bar_item_names[GUI_BAR_ITEM_LAG],
+                      &gui_bar_item_lag_cb, NULL, NULL);
+    gui_bar_item_hook_signal ("window_switch;buffer_switch;buffer_localvar_*",
+                              gui_bar_item_names[GUI_BAR_ITEM_LAG]);
+
     /* away message */
     gui_bar_item_new (NULL,
                       gui_bar_item_names[GUI_BAR_ITEM_AWAY],
@@ -2594,14 +2643,14 @@ gui_bar_item_print_log ()
     for (ptr_item = gui_bar_items; ptr_item; ptr_item = ptr_item->next_item)
     {
         log_printf ("");
-        log_printf ("[bar item (addr:0x%lx)]", ptr_item);
-        log_printf ("  plugin . . . . . . . . : 0x%lx ('%s')",
+        log_printf ("[bar item (addr:%p)]", ptr_item);
+        log_printf ("  plugin . . . . . . . . : %p ('%s')",
                     ptr_item->plugin, plugin_get_name (ptr_item->plugin));
-        log_printf ("  name . . . . . . . . . : '%s'",  ptr_item->name);
-        log_printf ("  build_callback . . . . : 0x%lx", ptr_item->build_callback);
-        log_printf ("  build_callback_pointer : 0x%lx", ptr_item->build_callback_pointer);
-        log_printf ("  build_callback_data. . : 0x%lx", ptr_item->build_callback_data);
-        log_printf ("  prev_item. . . . . . . : 0x%lx", ptr_item->prev_item);
-        log_printf ("  next_item. . . . . . . : 0x%lx", ptr_item->next_item);
+        log_printf ("  name . . . . . . . . . : '%s'", ptr_item->name);
+        log_printf ("  build_callback . . . . : %p", ptr_item->build_callback);
+        log_printf ("  build_callback_pointer : %p", ptr_item->build_callback_pointer);
+        log_printf ("  build_callback_data. . : %p", ptr_item->build_callback_data);
+        log_printf ("  prev_item. . . . . . . : %p", ptr_item->prev_item);
+        log_printf ("  next_item. . . . . . . : %p", ptr_item->next_item);
     }
 }
