@@ -21,13 +21,15 @@
 
 #include "CppUTest/TestHarness.h"
 
+#include "tests/tests.h"
+
 extern "C"
 {
 #include <string.h>
 #include <time.h>
 #include <sys/time.h>
-#include "src/core/wee-config.h"
-#include "src/core/wee-string.h"
+#include "src/core/core-config.h"
+#include "src/core/core-string.h"
 #include "src/gui/gui-buffer.h"
 #include "src/gui/gui-chat.h"
 #include "src/gui/gui-color.h"
@@ -210,6 +212,29 @@ TEST(GuiLine, BuildStringPrefixMessage)
     WEE_BUILD_STR_PREFIX_MSG("blue prefix\tred message", str_prefix, str_message);
 }
 
+/*
+ * Tests functions:
+ *   gui_line_build_string_message_nick_offline
+ */
+
+TEST(GuiLine, BuildStringMessageNickOffline)
+{
+    char *str, str_msg_expected[256], str_msg[256];
+
+    POINTERS_EQUAL(NULL, gui_line_build_string_message_nick_offline (NULL));
+    WEE_TEST_STR("", gui_line_build_string_message_nick_offline (""));
+
+    snprintf (str_msg_expected, sizeof (str_msg_expected),
+              "%stest",
+              GUI_COLOR(GUI_COLOR_CHAT_NICK_OFFLINE));
+    WEE_TEST_STR(str_msg_expected, gui_line_build_string_message_nick_offline ("test"));
+
+    snprintf (str_msg, sizeof (str_msg),
+              "%stest",
+              gui_color_get_custom ("blue"));
+    WEE_TEST_STR(str_msg_expected, gui_line_build_string_message_nick_offline (str_msg));
+}
+
 /*
  * Tests functions:
  *   gui_line_build_string_message_tags
@@ -527,6 +552,16 @@ TEST(GuiLine, HasOfflineNick)
     /* TODO: write tests */
 }
 
+/*
+ * Tests functions:
+ *   gui_line_is_action
+ */
+
+TEST(GuiLine, IsAction)
+{
+    /* TODO: write tests */
+}
+
 /*
  * Tests functions:
  *   gui_line_compute_buffer_max_length
@@ -854,7 +889,7 @@ TEST(GuiLine, New)
     gettimeofday (&date_printed, NULL);
     date.tv_sec = date_printed.tv_sec - 1;
     date.tv_usec = date_printed.tv_usec;
-    str_time = gui_chat_get_time_string (date.tv_sec, date.tv_usec);
+    str_time = gui_chat_get_time_string (date.tv_sec, date.tv_usec, 0);
 
     POINTERS_EQUAL(NULL,
                    gui_line_new (
